#ifndef HARDWARE_H_
#define HARDWARE_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const Hardware_h_Version =
    "$Id: Hardware.h,v 1.9 2000/12/18 23:42:36 bko Exp $";




/**********************************************************************
Filename   : hardware.h
Description: Specifies the MGCP gateway hardware API. 
             Defines a basic level of service provided by all
             supported hardware interfaces.
**********************************************************************/

#include <string>
#include <queue>


// events from the hardware to the control software
enum HardwareEvent
{
    EventNULL,
    EventOffHook,
    EventOnHook,
    EventDigit0,
    EventDigit1,
    EventDigit2,
    EventDigit3,
    EventDigit4,
    EventDigit5,
    EventDigit6,
    EventDigit7,
    EventDigit8,
    EventDigit9,
    EventDigitStar,
    EventDigitHash,
    EventAudioFailed,
    EventAudioClosed
};

// signals from the control software to the hardware
enum HardwareSignal
{
    SignalNULL,
    SignalRingStart,
    SignalRingStop,
    SignalRemoteRingbackStart,
    SignalRemoteRingbackStop,
    SignalLocalRingbackStart,
    SignalLocalRingbackStop,
    SignalBusyStart,
    SignalBusyStop,
    SignalAudioStop
};


/// Class: Hardware
/// Abstract base class for all hardware
class Hardware
{
    public:
        /// default constructor
        Hardware (void) : myDevice("")
        {}
        ;

        /// construct a hardware device object with specified device name
        Hardware (char* device) : myDevice(device)
        {}
        ;

        /// destructor
        virtual ~Hardware()
        {}
        ;

        /// add device's socket to the fd
        /// returns 0 if successful, errorcode otherwise
        virtual int addToFdSet (fd_set* fd) = 0;

        /// process all pending hardware events, some of which
        /// may require additonal processing by control software.
        /// returns the number of events requiring control software
        /// processing
        virtual int process (fd_set* fd) = 0;

        /// get hardware event requiring control software intervention.
        /// returns first pending hardware event, EventNULL if none
        virtual HardwareEvent getEvent() = 0;

        /// trigger the device to send the specified signal
        /// returns 0 if successful, errorcode otherwise
        virtual int sendSignal (HardwareSignal signal) = 0;

        /// start audio channel on device
        /// returns 0 if successful, errorcode otherwise
        virtual int audioStart (int in_port, const char* out_host, int out_port) = 0;

    protected:
        string myDevice;
        int myFD;  // file descriptor for device
        std::queue < HardwareEvent > outgoingEvents;
        std::queue < HardwareSignal > incomingSignals;
};

#endif
