#ifndef MGCP_COMMAND_HXX
#define MGCP_COMMAND_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcpCommand_hxx_Version =
    "$Id: MgcpCommand.hxx,v 1.38.4.1 2001/02/13 21:38:55 medala Exp $";


#include "global.h"
#include <string>
#include <deque>
#include <vector>
#include <typeinfo>
#include <map>
#include <set>
#include "UdpStack.hxx"
#include "SdpSession.hxx"
#include "MgcpResponseCode.hxx"
#include "MgcpParameter.hxx"
#include "MgcpParameterEventGeneric.hxx"
#include "Sptr.hxx"
#include "TokenRecognizer.hxx"
#include "NetworkAddress.h"
#include <iostream>


/**********************************************************************
 
 libmgcp
 
 Low-level MGCP library in C++.
 
 Basic architecture:
 
 In this stack, the caller should create objects for each command, as
 well as objects for each parameter.
 
 Each parameter be passed to the insert() function for the command.
 
 After all parameters have been inserted, then the send() function
 will send the packet over the wire to the receiver.
 
**********************************************************************/

#define DEFAULT_MGCP_PORT_FOR_CALLAGENT 2727
#define DEFAULT_MGCP_PORT_FOR_GATEWAY 2427
#define UDP_APPL_MIN_PORT_NUM 1024
#define UDP_APPL_MAX_PORT_NUM 65536

#define LONG_TIMER 30000   // 30 secs for LONG_TIMER in supporting at-most-once functionality

class UniqueId;
class MgcpPacket;
class MgcpCommand;
extern ostream& operator<<(ostream& s, const UniqueId& x);
extern ostream& operator<<(ostream& s, MgcpPacket& x);
extern ostream& operator<<(ostream& s, MgcpCommand& x);

/// forward declaration
class MgcpReceiver;

/// class for an unique id of an MGCP entity based on host:port:transactionID
class UniqueId
{
    public:

        /// default constructor
        UniqueId(): m_host(""),
        m_port( -1),
        m_transactionID( -1)
        { }


        /// constructor

        UniqueId(const string& host, int port, int transactionID)
        : m_host(host),
        m_port(port),
        m_transactionID(transactionID)
        { }

        /// copy constructor

        UniqueId(const UniqueId& uID)
        {
            m_host = uID.m_host;
            m_port = uID.m_port;
            m_transactionID = uID.m_transactionID;
        }


        string& getHost()
        {
            return m_host;
        }

        int getPort()
        {
            return m_port;
        }

        int getTransId ()
        {
            return m_transactionID;
        }


        /// less than operator < for map sorting
        bool operator <(const UniqueId& uID) const;

        /// equal comparison operator == for hash_map
        bool operator == (const UniqueId& uID) const;


        friend ostream& operator<<(ostream& s, const UniqueId& x);


    private:
        /// hostname of the entity
        string m_host;

        /// port on the host
        int m_port;

        /// transaction ID
        int m_transactionID;
};

/// class to hold a single RTP packet
class MgcpPacket
{
        // Curently header extentions are not supported

    public:
        /// Create a packet and allocate data space on heap
        MgcpPacket(int payloadSize);

        /// Free space allocated on heap
        ~MgcpPacket();

        /// Get UDP messsage location
        char* getMessageLocation();

        /// Get UDP messsage length in bytes
        int getMessageSize();

        /// Set UDP messsage lenght in bytes
        void setMessageSize(int size);

        /// set the source network address of the packet, i.e. where it came from
        void setAddress( NetworkAddress& n)
        {
            sourceAddr = n;
        }

        /// get source network address of the packet
        NetworkAddress getAddress()
        {
            return sourceAddr;
        }


        /// Print minimal info about packet, type, destination, timestamp
        friend ostream& operator<<(ostream& s, MgcpPacket& x);

        /// Print minimal info about packet, type, destination, timestamp
        ostream& print( ostream& stream ) const;

        /// Print all info about a packet including hex version of payload data
        void printVerbose( ostream& stream );


    private:
        /// pointer to the the raw data for the packet (including header)
        char* data;

        /// size of all the raw data for the packet (including header)
        int size;

        /// allocated size of data buffer
        int allocatedSize;

        NetworkAddress sourceAddr;
};


/**
   This class provides the transmission side of a connection for an MGCP
   agent or gateway.  
 
*/
class MgcpTransmitter
{
    public:
        ///
        MgcpTransmitter( Sptr < MgcpReceiver > receiver = 0 );
        ///
        MgcpTransmitter( const char* remoteHost, int aPort, Sptr < MgcpReceiver > receiver = 0 );  // fix this hard coded const
        ///
        string& protocolVersion()
        {
            static string s(" mgcp 0.1");
            return s;
        }

        /// transmit the data
        void transmit( char* data, int length );

        /// Stack willnetAddressPtr free this packet when it is done with it so it must be alloacted on the heap
        void transmit( MgcpPacket* packet );

        /// will allocate a packet and fill in UDP header fields
        MgcpPacket* allocatePacket( int payloadLength );


        ///
        MgcpTransmitter& operator=( const MgcpTransmitter& x )
        {
            this->remoteAddr = x.remoteAddr;
            return ( *this );
        }


        NetworkAddress& getremoteAddr()
        {
            return remoteAddr;
        }


    private:

        /// my UDP stack
        UdpStack* myStack;
        /// should I free my stack?
        bool freeStack;

        /// Target network address
        NetworkAddress remoteAddr;

};


///
class MgcpReceiver
{
    public:
        /// Create an input stack and start listening on aproperate port
        MgcpReceiver( int port, int jitterBufferMs = 50 );  // fix these hard coded const

        /// Destructor
        MgcpReceiver::~MgcpReceiver();

        /// Get the next chunk of data from the stream. Returns bytes received.
        int receive( char* buffer, int bufferSize);

        /// return a new UDP packet, non-blocking, NULL if none available, user must free this packet
        MgcpPacket* receive();

        /// Set the stack so it generates an error if it received data from other hosts
        ///  Not implemented, why here ?
        void onlyAcceptPacketsFrom( char* hostName );

        /// add this file descriptor to a FD_SET for a select call to use
        void addToFdSet( fd_set* set );

        /// Check if this file descriptor is set in an FD_SET for checking after a select call
        bool checkIfSet( fd_set* set );
        int getFd()
        {
            return ( myStack->getSocketFD() );
        }

        UdpStack * getUdpStack()
        {
            return ( myStack );
        }

    private:

        /// my UDP stack
        UdpStack* myStack;
        /// should I free my stack?
        bool freeStack;

        /// read the next  packet off the network
        MgcpPacket* getPacket();
};


typedef enum MgcpCommandType
{
    MgcpCommandTypeUndefined,
    MgcpCommandTypeResponse,
    MgcpCommandTypeEndpointConfiguration,
    MgcpCommandTypeNotificationRequest,
    MgcpCommandTypeNotify,
    MgcpCommandTypeCreateConnection,
    MgcpCommandTypeModifyConnection,
    MgcpCommandTypeDeleteConnection,
    MgcpCommandTypeAuditEndpoint,
    MgcpCommandTypeAuditConnection,
    MgcpCommandTypeRestartInProgress
}MgcpCommandType;

/***********************************************************************
 
  MGCP Commands
 
 **********************************************************************/


/**
   this class manages getting useful "global" items, like transaction IDs.
*/

class MgcpSingleton
{
    public:
        static int getTransactionId();
        static MgcpSingleton* instance()
        {
            if (instance__ == 0)
            {
                instance__ = new MgcpSingleton;
            }
            return instance__;
        }
        static string& protocolVersion()
        {
#ifndef NCS_PROFILE
            static string s(" MGCP 0.1");
#else // NCS_PROFILE
            static string s(" MGCP 1.0 NCS 1.0");
#endif 
            return s;
        }

    protected:
        MgcpSingleton() : myTransactionId(1)
        {}
        ;

        ~MgcpSingleton()
        {
            instance__ = 0;
        }

    private:
        static MgcpSingleton* instance__;
        int myTransactionId;

};


/**
  Abstract base class for all MGCP commands.
 
  All MGCP commands inherit from the MgcpCommand base class which
  provides the basics of the API.
 
  The most relevant functions include send() and insert.
 
  The current version of the MGCP command code contains a fair amount
  of redundant code which will probably be structured away.
 
*/
class MgcpCommand
{
    public:
        ///
        MgcpCommand() : message(""), myTarget(0)
        {}
        ///

        virtual ~MgcpCommand();

        ///
        virtual string encodeToSend() = 0;

        ///
        virtual bool send(MgcpTransmitter& target) = 0;

        ///
        virtual bool readyToSend() = 0;

        ///
        void setTarget(MgcpTransmitter* target)
        {
            myTarget = target;
        };

        ///
        void setTransactionId(int transId)
        {
            transactionId = transId ;
        }
        ///
        int getTransactionId()
        {
            return transactionId;
        }

        ///
        string getMessage()
        {
            return message;
        }

        ///
        void setMessage( string s )
        {
            message = s;
        }

        ///
        void setEndpointId(const MgcpParameterEndpointId& id)
        {
            endId = id;
        }
        ///
        const MgcpParameterEndpointId& getEndpointId()
        {
            return endId;
        }

        ///      insert the MgcpParameter if appropriate
        virtual void insert(Sptr < MgcpParameter > parameter);

        ///      insert the MgcpParameter if appropriate
        virtual void insert(MgcpParameter* parameter);

        ///
        virtual void printData(ostream& output);

        ///
        friend ostream& operator<<(ostream& s, MgcpCommand& x);

        ///
        virtual MgcpCommandType getType()
        {
            return MgcpCommandTypeUndefined;
        }

    protected:
        ///
        string message;
        ///
        int transactionId;
        ///
        MgcpParameterEndpointId endId;
        ///
        MgcpTransmitter* myTarget;
};


///  MGCP command acknowledgements are also encoded as MgcpCommand
class MgcpResponse : public MgcpCommand
{
    public:
        ///
        MgcpResponse()
        {}
        ;

        ///
        MgcpResponse(MgcpResponseCode responseCode, int transId, const string& comment)
        :
        myResponseCode(responseCode),
        myTransId(transId),
        myComment(comment)
        {}
        ;

        ///
        MgcpResponse(const MgcpResponse& x)
        :
        myResponseCode(x.myResponseCode),
        myTransId(x.myTransId),
        myComment(x.myComment),
        parameters(x.parameters)
        {}
        ;

        ///
        ~MgcpResponse()
        {}

        ///

        bool readyToSend();

        ///
        string encodeToSend();

        ///
        bool send(MgcpTransmitter& target);

        ///
        void insert(Sptr < MgcpParameter > parameter)
        {
            // check for if this was inserted ?
            parameters[parameter->type()] = parameter;
        }

        ///
        void insert(MgcpParameter* parameter)
        {
            // check for if this was inserted ?
            parameters[parameter->type()] = Sptr < MgcpParameter > (parameter);
        }

        ///
        void printData(ostream& output);

        ///
        void setResponseCode(MgcpResponseCode x)
        {
            myResponseCode = x;
        }
        ///
        MgcpResponseCode getResponseCode()
        {
            return myResponseCode;
        }

        ///
        void setComment(const string& x)
        {
            myComment = x;
        }
        ///
        string getComment()
        {
            return myComment;
        }

        ///
        MgcpParameterLocalConnectionDescriptor* getLocalConnect()
        {
            return
                dynamic_cast < MgcpParameterLocalConnectionDescriptor* >
                (parameters[MgcpParameterTypeLocalConnectionDescriptor].getPtr());
        }

        ///
        MgcpCommandType getType()
        {
            return MgcpCommandTypeResponse;
        }

        ///
        MgcpResponse& operator= (const MgcpResponse& id)
        {
            this->myResponseCode = id.myResponseCode;
            this->myTransId = id.myTransId;
            this->myComment = id.myComment;
            this->parameters = id.parameters;

            return *this;
        }

        ///
        MgcpParameter* retrieve(MgcpParameterType parm)
        {
            return parameters[parm].getPtr();
        }

        ///
        Sptr < MgcpParameter > retrieveParam(MgcpParameterType parm)
        {
            return parameters[parm];
        }

        ///
        void clear(MgcpParameterType parm)
        {
            Sptr < MgcpParameter > p = parameters[parm];
            if (p != 0)
            {
                p = 0;
                parameters.erase(parameters.find(parm));
            }
        }

    private:
        ///
        MgcpResponseCode myResponseCode;
        ///
        int myTransId;
        ///
        string myComment;
        /// this map contains the parameters for an MgcpResponse
        map < MgcpParameterType, Sptr < MgcpParameter > > parameters;
};


///
class MgcpRequest: public MgcpCommand
{
    public:
        ///
        MgcpRequest()
        {}

        ///
        ~MgcpRequest()
        {}

        ///
        bool readyToSend();

        ///
        string encodeToSend();

        ///
        bool send(MgcpTransmitter& target);

        ///
        void insert(Sptr < MgcpParameter > parameter)
        {
            // check for if this was inserted ?
            parameters[parameter->type()] = parameter;
        }

        ///
        void insert(MgcpParameter* parameter)
        {
            // check for if this was inserted ?
            parameters[parameter->type()] = Sptr < MgcpParameter > (parameter);
        }

        ///
        void printData(ostream& output);

        ///
        virtual const string& getVerb() = 0;

        ///
        Sptr < MgcpParameter > retrieveParam(MgcpParameterType parm)
        {
            return parameters[parm];
        }

        ///
        MgcpParameter* retrieve(MgcpParameterType parm)
        {
            return parameters[parm].getPtr();
        }

        ///
        void clear(MgcpParameterType parm)
        {
            Sptr < MgcpParameter > p = parameters[parm];
            if (p != 0)
            {
                p = 0;
                parameters.erase(parameters.find(parm));
            }
        }

    private:
        ///
        MgcpRequest(const MgcpRequest& x)
        : parameters(x.parameters)
        {}
        
        ///
        MgcpRequest& operator=(const MgcpRequest& );
        
    protected:
        /// this map contains the parameters for any MgcpRequest
        map < MgcpParameterType, Sptr < MgcpParameter > > parameters;
};


///
class MgcpCommandEndpointConfiguration: public MgcpRequest
{
    public:
        ///
        MgcpCommandEndpointConfiguration()
        {}
        ;

        ///
        const string& getVerb()
        {
            return myVerb();
        }

        ///
        static const string& myVerb()
        {
            //	static const string verb("epcf"); return verb;
            static const string verb("EPCF");
            return verb;
        }

        ///
        MgcpCommandType getType()
        {
            return MgcpCommandTypeEndpointConfiguration;
        }

    private:

        /// only declared, not defined
        MgcpCommandEndpointConfiguration( const MgcpCommandEndpointConfiguration& );
  
        /// onle declared, not defined
        MgcpCommandEndpointConfiguration& operator=( const MgcpCommandEndpointConfiguration&);

};


///
class MgcpCommandNotificationRequest: public MgcpRequest
{
    public:
        ///
        MgcpCommandNotificationRequest()
        {}
        ;

        ///
        const string& getVerb()
        {
            return myVerb();
        }

        ///
        static const string& myVerb()
        {
            //	static const string verb("rqnt"); return verb;
            static const string verb("RQNT");
            return verb;
        }

        ///
        MgcpCommandType getType()
        {
            return MgcpCommandTypeNotificationRequest;
        }

    private:
        /// only declared, not defined
        MgcpCommandNotificationRequest( const MgcpCommandNotificationRequest&);

        /// only declared, not defined
        MgcpCommandNotificationRequest& operator=( const MgcpCommandNotificationRequest&);

};


///
class MgcpCommandNotify: public MgcpRequest
{
    public:
        ///
        MgcpCommandNotify()
        {}
        ;

        ///
        const string& getVerb()
        {
            return myVerb();
        }

        ///
        static const string& myVerb()
        {
            //	static const string verb("ntfy"); return verb;
            static const string verb("NTFY");
            return verb;
        }

        ///
        MgcpCommandType getType()
        {
            return MgcpCommandTypeNotify;
        }

    private:
        /// only declared, not defined
        MgcpCommandNotify( const MgcpCommandNotify& );

        /// only declared, not defined
        MgcpCommandNotify& operator=( const MgcpCommandNotify& );

};


///
class MgcpCommandCreateConnection: public MgcpRequest
{
    public:
        ///
        MgcpCommandCreateConnection()
        {}
        ;

        ///
        const string& getVerb()
        {
            return myVerb();
        }

        ///
        static const string& myVerb()
        {
            //	static const string verb("crcx"); return verb;
            static const string verb("CRCX");
            return verb;
        }

        ///
        MgcpCommandType getType()
        {
            return MgcpCommandTypeCreateConnection;
        }

    private:
        /// only declared, not defined
        MgcpCommandCreateConnection( const MgcpCommandCreateConnection& );
  
        /// only declared, not defined
        MgcpCommandCreateConnection& operator=( const MgcpCommandCreateConnection& );

};



///
class MgcpCommandModifyConnection: public MgcpRequest
{
    public:
        ///
        MgcpCommandModifyConnection()
        {}
        ;

        ///
        const string& getVerb()
        {
            return myVerb();
        }

        ///
        static const string& myVerb()
        {
            //	static const string verb("mdcx"); return verb;
            static const string verb("MDCX");
            return verb;
        }

        ///
        MgcpCommandType getType()
        {
            return MgcpCommandTypeModifyConnection;
        }

    private:
        /// only declared, not defined
        MgcpCommandModifyConnection( const MgcpCommandModifyConnection& );
  
        ///
        MgcpCommandModifyConnection& operator=( const MgcpCommandModifyConnection& );

};


///
class MgcpCommandDeleteConnection: public MgcpRequest
{
    public:
        ///
        MgcpCommandDeleteConnection()
        {}
        ;

        ///
        const string& getVerb()
        {
            return myVerb();
        }

        ///
        static const string& myVerb()
        {
            //	static const string verb("dlcx"); return verb;
            static const string verb("DLCX");
            return verb;
        }

        ///
        MgcpCommandType getType()
        {
            return MgcpCommandTypeDeleteConnection;
        }

    private:
        /// only declared, not defined
        MgcpCommandDeleteConnection( const MgcpCommandDeleteConnection& );
  
        /// only declared, not defined
        MgcpCommandDeleteConnection& operator=( const MgcpCommandDeleteConnection& );

};

typedef MgcpCommandDeleteConnection MgcpCommandDeleteConnectionAgent;
typedef MgcpCommandDeleteConnection MgcpCommandDeleteConnectionGw;


///
class MgcpCommandAuditEndpoint: public MgcpRequest
{
    public:
        ///
        MgcpCommandAuditEndpoint()
        {}
        ;

        ///
        const string& getVerb()
        {
            return myVerb();
        }

        ///
        static const string& myVerb()
        {
            //	static const string verb("auep"); return verb;
            static const string verb("AUEP");
            return verb;
        }

        ///
        MgcpCommandType getType()
        {
            return MgcpCommandTypeAuditEndpoint;
        }

    private:
        /// only declared, not defined
        MgcpCommandAuditEndpoint( const MgcpCommandAuditEndpoint& );
  
        /// only declared, not defined
        MgcpCommandAuditEndpoint& operator=( const MgcpCommandAuditEndpoint&);

};



///
class MgcpCommandAuditConnection: public MgcpRequest
{
    public:
        ///
        MgcpCommandAuditConnection()
        {}
        ;

        ///
        const string& getVerb()
        {
            return myVerb();
        }

        ///
        static const string& myVerb()
        {
            //	static const string verb("aucx"); return verb;
            static const string verb("AUCX");
            return verb;
        }

        ///
        MgcpCommandType getType()
        {
            return MgcpCommandTypeAuditConnection;
        }

    private:
        /// only declared, not defined
        MgcpCommandAuditConnection( const MgcpCommandAuditConnection& );

        /// only declared, not defined
        MgcpCommandAuditConnection& operator=( const MgcpCommandAuditConnection& );

};



///
class MgcpCommandRestartInProgress: public MgcpRequest
{
    public:
        ///
        MgcpCommandRestartInProgress()
        {}
        ;

        ///
        const string& getVerb()
        {
            return myVerb();
        }

        ///
        static const string& myVerb()
        {
            // static const string verb("rsip"); return verb;
            static const string verb("RSIP");
            return verb;
        }

        ///
        MgcpCommandType getType()
        {
            return MgcpCommandTypeRestartInProgress;
        }

    private:
        /// only declared, not defined
        MgcpCommandRestartInProgress( const MgcpCommandRestartInProgress& );

        /// only declared, not defined
        MgcpCommandRestartInProgress& operator=( const MgcpCommandRestartInProgress& );

};

#endif
