#ifndef MGCP_PARAMETER_HXX
#define MGCP_PARAMETER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MgcpParameter_hxx_Version =
"$Id: MgcpParameter.hxx,v 1.46 2001/02/01 01:22:56 icahoon Exp $";


#include "global.h"
#include <string>
#include <string.h>
#include <vector>
#include <deque>
#include <typeinfo>
#include "SdpSession.hxx"
#include "MgcpBaseEnums.hxx"
#include "Sptr.hxx"
#include "MgcpConnectionOptions.hxx"
#include "MgcpPackageTypes.hxx"
#include "support.hxx"


typedef split_t ParameterSplitType;

#define MAX_ENDPOINT_ID_LIST 16


/***********************************************************************
   MGCPParameter
 
   These are the parameters which can be inserted from the commands.
   All of them inherit from the base class MgcpParameter .
 **********************************************************************/


/** parameter types for the parameters.  Useful for the MgcpCommands
    because we are going to a map<MgcpParameterType, MgcpParameter*> to
    store the parameters.
*/
typedef enum MgcpParameterType
{
    MgcpParameterTypeUnknownParameter,
    MgcpParameterTypeQuarantineHandling,
    MgcpParameterTypeResponseAcknowledgement,
    MgcpParameterTypeEndpointId,
    MgcpParameterTypeSecondEndpointId,
    MgcpParameterTypeSpecificEndpointId,
    MgcpParameterTypeRequestedInfo,
    MgcpParameterTypeEndpointIdList,
    MgcpParameterTypeBearerInformation,
    MgcpParameterTypeNotifiedEntity,
    MgcpParameterTypeEvent,
    MgcpParameterTypeObservedEvents,
    MgcpParameterTypeConnectionParameters,
    MgcpParameterTypeReasonCode,
    MgcpParameterTypeRequestedEvents,
    MgcpParameterTypeSignal,
    MgcpParameterTypeSignalRequests,
    MgcpParameterTypeRequestId,
    MgcpParameterTypeCallId,
    MgcpParameterTypeConnectionId,
    MgcpParameterTypeMaxEndPointIds,
    MgcpParameterTypeNumEndPoints,
    MgcpParameterTypeDigitMap,
    MgcpParameterTypeDetectEvents,
    MgcpParameterTypeLocalConnectionOptions,
    MgcpParameterTypeConnectionMode,
    MgcpParameterTypeLocalConnectionDescriptor,
    MgcpParameterTypeRemoteConnectionDescriptor,
    MgcpParameterTypeRestartMethod,
    MgcpParameterTypeRestartDelay,
    MgcpParameterTypeCapabilities,
    MgcpParameterTypeEventStates,
    MgcpParameterTypeMaxValue  // this one MUST be last
}MgcpParameterType;


class MgcpParameter;
extern ostream& operator<<(ostream& s, const MgcpParameter& x);


/**
    MgcpParameter is an abstract base class for MGCP parameters (the
    arguments to commands).
 
    All of the parameters inherit from MgcpParameter.
 
    The key function for parameters is encode(), which encodes the
    parameter for inclusion in the MGCP command.
*/

class MgcpParameter
{
    public:
        ///
        MgcpParameter() {}

        ///
        virtual ~MgcpParameter();
        /**
        returns a pointer to a string which contains the MGCP text encoding
        of the parameter.

        In a future release, encode may return a char* to improve efficiency.
        */
        virtual string encode();
        ///

        friend ostream& operator<<(ostream& s, const MgcpParameter& x);

        ///
        virtual ostream& printData(ostream& s) const
        {
            return s << "(not encoded)";
        }

        ///
        virtual MgcpParameter* duplicate();

        /** the type of the Parameter.  Used instead of the type_info because
        it appars that type_info is less flexible. 
        */
        virtual MgcpParameterType type()
        {
            return MgcpParameterTypeUnknownParameter;
        }

    private:
        /// only declared, not defined
        MgcpParameter( const MgcpParameter& );
  
        /// only declared, not defined
        MgcpParameter& operator=( const MgcpParameter& );

};


///
class MgcpParameterResponseAcknowledgement : public MgcpParameter
{
    public:
        ///
        MgcpParameterResponseAcknowledgement()
        {}
        

        ///
        MgcpParameterResponseAcknowledgement
        (const MgcpParameterResponseAcknowledgement& x)
        : myResponse(x.myResponse)
        {}
        

        ///
        MgcpParameterResponseAcknowledgement(const string& foo)
        : myResponse(foo)
        {}
        

        ///
        ~MgcpParameterResponseAcknowledgement() 
        {}

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeResponseAcknowledgement;
        }
        ///
        string encode()
        {
            string y;
            y = "K: " + myResponse + "\r\n";
            return y;
        }

        bool parse( const ParameterSplitType& parms )
        {
            if (parms.size() > 1)
            {
                myResponse = parms[1];
                return true;
            }
            return false;
        }

        ///
        string getDecoded()
        {
            return myResponse;
        }


    private:
        /// only declared, not defined
        MgcpParameterResponseAcknowledgement& operator=( const MgcpParameterResponseAcknowledgement& );

        ///
        string myResponse;
};


///
class MgcpParameterEndpointId : public MgcpParameter
{
    public:
        ///
        MgcpParameterEndpointId()
        {}
        
        ///
        MgcpParameterEndpointId(const MgcpParameterEndpointId& x) :
        myId(x.myId)
        {}

        ///
        MgcpParameterEndpointId(const string& id)
        {
            myId = id;
        }

        ///
        ~MgcpParameterEndpointId()
        {}

        ///
        string encode()
        {
            return myId;
        }
        ///
        MgcpParameterEndpointId& operator= (const MgcpParameterEndpointId& id)
        {
            this->myId = id.myId;
            return *this;
        }
        ///
        int operator== (const MgcpParameterEndpointId& id) const
        {
            return myId == id.myId;
        }

        ///
        void print()
        {
            cout << myId;
        }
        ///
        ostream& printData(ostream& s) const
        {
            return s << myId;
        };

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterEndpointId(*this);
        };

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeEndpointId;
        }

        ///
        string getDecoded()
        {
            return myId;
        }

    private:
        ///
        string myId;
};


///
class MgcpParameterSecondEndpointId : public MgcpParameter
{
    public:
        ///
        MgcpParameterSecondEndpointId()
        {}

        ///
        MgcpParameterSecondEndpointId(const MgcpParameterSecondEndpointId& x) :
        myId(x.myId)
        { }

        ///
        ~MgcpParameterSecondEndpointId()
        {}

        ///
        MgcpParameterSecondEndpointId(const string& id)
        {
            myId = id;
        }
        ///
        string encode()
        {
            return myId;
        }
        ///
        MgcpParameterSecondEndpointId& operator= (const MgcpParameterSecondEndpointId& id)
        {
            this->myId = id.myId;
            return *this;
        }
        ///
        int operator== (const MgcpParameterSecondEndpointId& id) const
        {
            return myId == id.myId;
        }

        ///
        void print()
        {
            cout << myId;
        }
        ///
        ostream& printData(ostream& s) const
        {
            return s << myId;
        }

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterSecondEndpointId(*this);
        }

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeSecondEndpointId;
        }

        ///
        string getDecoded()
        {
            return myId;
        }

    private:
        ///
        string myId;
};


class MgcpParameterRequestedInfo : public MgcpParameter
{
    public:
        ///
        MgcpParameterRequestedInfo()
        {}
          
        ///
        MgcpParameterRequestedInfo(const MgcpParameterRequestedInfo& r) : list(r.list)
        {}
        ///

        ~MgcpParameterRequestedInfo()
        {
	    list.clear();
	}

        ///

        string encode();

        ///
        bool insert( MgcpParameterType type );
        ///

        ///
        ostream& printData(ostream& s);


        ///
        MgcpParameter* duplicate()
        {
            return ( new MgcpParameterRequestedInfo( *this ) );
        }

        ///
        bool parse( const ParameterSplitType& parms);
        ///
        vector < MgcpParameterType > * getList()
        {
            return &list;
        }

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeRequestedInfo;
        }

        int getDecodedNum()
        {
            return list.size();
        }
        ///
        MgcpParameterType getDecodedItem(int i)
        {
            assert(static_cast < unsigned int > (i) < list.size());
            return list[i];
        }

    private:
        /// only declared, not defined
        MgcpParameterRequestedInfo& operator=( const MgcpParameterRequestedInfo& );

        ///
        string encodeParamType(MgcpParameterType);
        ///
        MgcpParameterType decodeParamType( string s );

        ///
        vector < MgcpParameterType > list;
};


class MgcpParameterEndpointIdList : public MgcpParameter
{
    public:
        ///
        MgcpParameterEndpointIdList()
        {}

        ///
        MgcpParameterEndpointIdList(const MgcpParameterEndpointIdList& e) : list(e.list)
        {}
        
        ///
        ~MgcpParameterEndpointIdList()
        {
	    list.clear();
	}
        

        ///
        string encode();

        ///
        bool insert(Sptr < MgcpParameterEndpointId > id);


        ///
        ostream& printData(ostream& s) const;

        MgcpParameter* duplicate()
        {
            return new MgcpParameterEndpointIdList(*this);
        }
        ///
        bool parse(const ParameterSplitType& parms);

        ///
        vector < Sptr < MgcpParameterEndpointId > > * getList()
        {
            return &list;
        }
        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeEndpointIdList;
        }

        ///
        int getDecodedNum()
        {
            return list.size();
        }
        ///
        MgcpParameterEndpointId* getDecodedItem(int i)
        {
            assert(static_cast < unsigned int > (i) < list.size());
            return list[i].getPtr();
        }

    private:
        /// only declared, not defined
        MgcpParameterEndpointIdList& operator=( const MgcpParameterEndpointIdList& );

        ///
        vector < Sptr < MgcpParameterEndpointId > > list;

};


///
typedef enum EncodingMethod
{
    EncodingMethodInvalid,
    EncodingMethodALaw,
    EncodingMethoduLaw

}EncodingMethod;


///
class MgcpParameterBearerInformation : public MgcpParameter
{
    public:
        ///
        MgcpParameterBearerInformation()
	  :encodingMethod( EncodingMethodInvalid )
        {}

        ///
        MgcpParameterBearerInformation(const MgcpParameterBearerInformation& info)
        {
            encodingMethod = info.encodingMethod;
        }

        ///
        MgcpParameterBearerInformation(EncodingMethod encoding = EncodingMethoduLaw)
        {
            encodingMethod = encoding;
        }

        ///
        MgcpParameterBearerInformation(const MgcpBearerInformation& x);

        ///
        ~MgcpParameterBearerInformation()
        {}

        ///
        string encode()
        {
            static string aLaw("B: e:a\r\n");
            static string muLaw("B: e:mu\r\n");

            if (encodingMethod == EncodingMethodALaw)
            {
                return aLaw;
            }
            else
            {
                return muLaw;
            }
        }

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterBearerInformation(*this);
        }
        ///
        ostream& printData(ostream& s) const
        {
            if (encodingMethod == EncodingMethodALaw)
            {
                return s << "A-Law";
            }
            else
            {
                return s << "mu-Law";
            }
        }
        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeBearerInformation;
        }

        ///
        MgcpBearerInformation getDecoded()
        {
            if (encodingMethod == EncodingMethodALaw)
            {
                return MgcpBearerInformationEnumALaw;
            }
            else
            {
                return MgcpBearerInformationEnumMuLaw;
            }
        }

    private:
        /// only declared, not defined
        MgcpParameterBearerInformation& operator=( const MgcpParameterBearerInformation& );

        ///
        EncodingMethod encodingMethod;
};


///
class MgcpParameterNotifiedEntity : public MgcpParameter
{
    public:
        ///
        MgcpParameterNotifiedEntity()
        {}
        
        ///
        MgcpParameterNotifiedEntity(char* name) : myEntityName(name)
        {}
        
        ///
        MgcpParameterNotifiedEntity(const string& name) : myEntityName(name)
        {}
        
        ///
        MgcpParameterNotifiedEntity(const MgcpParameterNotifiedEntity& x) : myEntityName(x.myEntityName)
        {}
        

        ///
        ~MgcpParameterNotifiedEntity()
        {}

        ///
        string encode();
        ///
        ostream& printData(ostream& s) const
        {
            return s << myEntityName;
        }
        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterNotifiedEntity(*this);
        }
        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeNotifiedEntity;
        }

        ///
        string getDecoded()
        {
            return myEntityName;
        }
    private:
        /// only declared, not defined
        MgcpParameterNotifiedEntity& operator=( const MgcpParameterNotifiedEntity& );

        ///
        string myEntityName;
};


/***********************************************************************
 
       Events
 
**********************************************************************/

/// Event is the abstract base class for events.
class MgcpParameterEvent : public MgcpParameter
{
    public:
        ///
        MgcpParameterEvent()
        {}
        
        ///
        MgcpParameterEvent(const MgcpParameterEvent& e)
        {}
        
        ///
        ~MgcpParameterEvent()
        {}

        ///
        virtual string encode()
        {
            assert(0);
            return string("bad event encoding");
        }

        ///
        ostream& printData(ostream& s) const
        {
            return s << "no event printed";
        }
        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterEvent(*this);
        }

    private:
        /// only declared, not defined
        MgcpParameterEvent& operator=( const MgcpParameterEvent& );

};


/// Event is the abstract base class for requested events.
class MgcpParameterRequestedEvent : public MgcpParameter
{
    public:
        ///
        MgcpParameterRequestedEvent()
        {}
        
        ///
        MgcpParameterRequestedEvent(const MgcpParameterRequestedEvent& e)
        {}
        
        ///
        ~MgcpParameterRequestedEvent()
        {}

        ///
        virtual string encode()
        {
            assert(0);
            return ("bad event encoding");
        }

        ///
        ostream& printData(ostream& s) const
        {
            return s << "no event printed";
        }
        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterRequestedEvent(*this);
        }

    private:
        /// only declared, not defined
        MgcpParameterRequestedEvent& operator=( const MgcpParameterRequestedEvent& );

};


/// Signal is an abstract base class for all signals.
class MgcpParameterSignal : public MgcpParameter
{
    public:
};



///
class MgcpParameterSignalRequests : public MgcpParameter
{
    public:
        ///
        MgcpParameterSignalRequests()
        {}
            
        ///
        ~MgcpParameterSignalRequests()
        {
            list.clear();
        }

        ///
        MgcpParameterSignalRequests(const MgcpParameterSignalRequests& e) : list(e.list)
        {}
        
        ///
        string encode();

        ///
        bool insert(Sptr < MgcpParameterSignal > event);

        ///
        ostream& printData(ostream& s) const;

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterSignalRequests(*this);
        }

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        vector < Sptr < MgcpParameterSignal > > * getList()
        {
            return &list;
        }

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeSignalRequests;
        }

        ///
        int getDecodedNum()
        {
            return list.size();
        }

        ///
        MgcpParameterSignal* getDecodedItem(int i)
        {
            assert(static_cast < unsigned int > (i) < list.size());
            return list[i].getPtr();
        }

    private:
        /// only declared, not defined
        MgcpParameterSignalRequests& operator=( const MgcpParameterSignalRequests& );

        ///
        vector < Sptr < MgcpParameterSignal > > list;
};


/***********************************************************************
 
			     Event Lists
 
**********************************************************************/

///
class MgcpParameterEventList : public MgcpParameter
{
    public:
        ///
        MgcpParameterEventList()
        {}
            
        /// 
        ~MgcpParameterEventList()
        {
            list.clear();
        }

        ///
        MgcpParameterEventList(const MgcpParameterEventList& e) : list(e.list)
        {}
        
        ///
        string encode();

        ///
        bool insert(Sptr < MgcpParameterEvent > event);

        ///
        ostream& printData(ostream& s) const;

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterEventList(*this);
        }

        ///
        vector < Sptr < MgcpParameterEvent > > * getVector()
        {
            return &list;
        }

        ///
        int getDecodedNum()
        {
            return list.size();
        }

        MgcpParameterEvent* getDecodedItem(int i)
        {
            assert(static_cast < unsigned int > (i) < list.size());
            return list[i].getPtr();
        }

    private:
        /// only declared, not defined
        MgcpParameterEventList& operator=( const MgcpParameterEventList& );

        ///
        vector < Sptr < MgcpParameterEvent > > list;
};


///
class MgcpParameterRequestedEvents : public MgcpParameterEventList
{
    public:
        ///
        MgcpParameterRequestedEvents()
        {}
        
        ///
        MgcpParameterRequestedEvents( const MgcpParameterRequestedEvents& other)
	  :requestList( other.requestList )
        {}

        ///
        ~MgcpParameterRequestedEvents()
        {
            requestList.clear();
        }

        ///
        string encode();

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterRequestedEvents(*this);
        }

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeRequestedEvents;
        }

        ///
        bool insert(Sptr < MgcpParameterRequestedEvent > event);
        
        ///
        vector < Sptr < MgcpParameterRequestedEvent > > * getVector()
        {
            return &requestList;
        }
        ///
        int getDecodedNum()
        {
            return requestList.size();
        }
        ///
        MgcpParameterRequestedEvent* getDecodedItem(int i)
        {
            assert(static_cast < unsigned int > (i) < requestList.size());
            return requestList[i].getPtr();
        }

        ///
        ostream& printData(ostream& s) const;

    private:
        /// only declared, not defined
        MgcpParameterRequestedEvents& operator=( const MgcpParameterRequestedEvents& );

        ///
        vector < Sptr < MgcpParameterRequestedEvent > > requestList;
};


///
class MgcpParameterObservedEvents : public MgcpParameterEventList
{
    public:
        ///
        MgcpParameterObservedEvents()
        {}
        
        ///
        MgcpParameterObservedEvents( const MgcpParameterObservedEvents& )
        {}

        ///
        ~MgcpParameterObservedEvents()
        {}

        ///
        string encode();

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterObservedEvents(*this);
        }

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeObservedEvents;
        }

    private:
        /// only declared, not defined
        MgcpParameterObservedEvents& operator=( const MgcpParameterObservedEvents& );

};


///
class MgcpParameterDetectEvents : public MgcpParameterEventList
{
    public:
        ///
        MgcpParameterDetectEvents()
        {}
        
        /// 
        MgcpParameterDetectEvents( const MgcpParameterDetectEvents& other)
        {}

        ///
        ~MgcpParameterDetectEvents()
        {}

        ///
        string encode();

        ///
        bool parse(const ParameterSplitType& parms);

        ///
  
        MgcpParameter* duplicate()
        {
            return new MgcpParameterDetectEvents(*this);
        }
  

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeDetectEvents;
        }


    private:
        /// only declared, not defined
        MgcpParameterDetectEvents& operator=( const MgcpParameterDetectEvents& );

};


/***********************************************************************
 
		   RequestId, CallId, ConnectionId
 
**********************************************************************/

///
class MgcpParameterRequestId : public MgcpParameter
{
    public:
        ///
        MgcpParameterRequestId()
        {}
        
        ///
        MgcpParameterRequestId(const string& s) : myId(s)
        {}
        
        ///
        MgcpParameterRequestId(char* s) : myId(s)
        {}
        
        ///
        MgcpParameterRequestId(const MgcpParameterRequestId& r) : myId(r.myId)
        {}
        
        ///
        ~MgcpParameterRequestId()
        {}

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterRequestId(*this);
        }

        ///
        MgcpParameterRequestId& operator= (const MgcpParameterRequestId& id)
        {
	    if ( this != &id )
	    {
                this->myId = id.myId;
	    }

            return *this;
        }

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeRequestId;
        }

        ///
        string getDecoded()
        {
            return myId;
        }

        ///
        string encode();

        ///
        bool parse(const ParameterSplitType& parms);

    private:
        ///
        string myId;
};


///
class MgcpParameterCallId : public MgcpParameter
{
    public:
        ///
        MgcpParameterCallId()
        {}
        
        ///
        MgcpParameterCallId(char* id) : myId(id)
        {}
        
        ///
        MgcpParameterCallId(const string& id) : myId(id)
        {}
        
        ///
        MgcpParameterCallId(const MgcpParameterCallId& id) : myId(id.myId)
        {}
        
        ///
        ~MgcpParameterCallId()
        {}

        ///
        string encode();

        ///
        ostream& printData(ostream& s) const
        {
            return s << myId;
        }

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterCallId(*this);
        }

        ///
        int operator== (const MgcpParameterCallId& id) const
        {
            return myId == id.myId;
        }

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeCallId;
        }

        ///
        string getDecoded()
        {
            return myId;
        }

    private:
        /// only declared, not defined
        MgcpParameterCallId& operator=( const MgcpParameterCallId& );

        ///
        string myId;
};


class MgcpParameterConnectionId : public MgcpParameter
{
    public:
        ///
        MgcpParameterConnectionId()
        {}

        ///
        MgcpParameterConnectionId(char* id)
        {
            myIdList.push_back(id);
        }

        ///
        MgcpParameterConnectionId(const string& id)
        {
            myIdList.push_back(id);
        }

        ///
        MgcpParameterConnectionId(const MgcpParameterConnectionId& idlist) : myIdList(idlist.myIdList)
        {}
        
        ///
        ~MgcpParameterConnectionId()
        {
	    myIdList.clear();
	}
        
        ///
        string encode();

        ///
        ostream& printData(ostream& s) const;

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterConnectionId(*this);
        }

        ///
        int operator== (const MgcpParameterConnectionId& idlist) const
        {
            return myIdList == idlist.myIdList;
        }

        ///
        bool insert(const string& type );

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeConnectionId;
        }

        ///
        vector < string > * getlist()
        {
            return &myIdList;
        }

        ///
        int getDecodedNum()
        {
            return myIdList.size();
        }

        ///
        string getDecoded()
        {
            return myIdList.front();
        }

        ///
        string getDecodedItem(int i)
        {
            assert(static_cast < unsigned int > (i) < myIdList.size());
            return myIdList[i];
        }

    private:
        /// only declared, not defined
        MgcpParameterConnectionId& operator=( const MgcpParameterConnectionId& );

        ///
        vector < string > myIdList;
};


/***************************************************
  MaxEndPointIds
*******************************************************/

class MgcpParameterMaxEndPointIds : public MgcpParameter
{
    public:
        ///
        MgcpParameterMaxEndPointIds()
	  :myId( 0 )
        {
	    st[0] = '\0';
	}

        ///
        //  MgcpParameterMaxEndPointIds(char* id) : myId(id) {};
        ///
        //  MgcpParameterMaxEndPointIds(const string& id) : myId(id) {};
        ///

        MgcpParameterMaxEndPointIds(int d)
	  :myId( d )
        {
            itoa(d, st);
        }

        ///
        MgcpParameterMaxEndPointIds(const MgcpParameterMaxEndPointIds& id)
	  :myId( id.myId )
        {
            strncpy( st, id.st, MAX_ENDPOINT_ID_LIST );
	    st[MAX_ENDPOINT_ID_LIST-1] = '\0';
        }

        ///
        ~MgcpParameterMaxEndPointIds()
        {}

        ///
        string encode();

        ///
        ostream& printData(ostream& s) const
        {
            return s << myId;
        }

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterMaxEndPointIds(*this);
        }

        ///
        int operator== (const MgcpParameterMaxEndPointIds& id) const
        {
            return myId == id.myId;
        }

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeMaxEndPointIds;
        }

        ///
        int getDecoded()
        {
            return atoi(st);
        }

        ///
        void itoa( int myId, char st[] );

        ///
        void reverse( char st[] );

        ///
        int atoi(char st[]);

    private:
        /// only declared, not defined
        MgcpParameterMaxEndPointIds& operator=( const MgcpParameterMaxEndPointIds& );

        ///
        int myId;

        ///
        char st[MAX_ENDPOINT_ID_LIST];
};


/*********************************************************
NumEndPoints
**********************************************************/

class MgcpParameterNumEndPoints : public MgcpParameter
{
    public:
        ///
        MgcpParameterNumEndPoints()
	  :num( 0 )
        {
            st[0] = '\0';
        }

        ///
        MgcpParameterNumEndPoints(int d)
	  :num( d )
        {
            itoa(d, st);
        }

        ///
        //    MgcpParameterNumEndPoints(char* id) : s(id) {};
        ///
        //   MgcpParameterNumEndPoints(const string& id) : s(id) {};
        ///

        MgcpParameterNumEndPoints(const MgcpParameterNumEndPoints& id)
	  :num( id.num )
        {
            strncpy(st, id.st, MAX_ENDPOINT_ID_LIST);
	    st[MAX_ENDPOINT_ID_LIST-1] = '\0';
        }

        ///
        ~MgcpParameterNumEndPoints()
        {}

        ///
        string encode();

        ///
        ostream& printData(ostream& s) const
        {
            return s << num;
        }

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterNumEndPoints(*this);
        }

        ///
        int operator== (const MgcpParameterNumEndPoints& id) const
        {
            return num == id.num && st == id.st;
        }

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeNumEndPoints;
        }

        ///
        int getDecoded()
        {
            return atoi(st);
        }

        ///
        void itoa( int num, char st[] );

        ///
        void reverse( char st[] );

        ///
        int atoi(char st[]);

    private:
        /// only declared, not defined
        MgcpParameterNumEndPoints& operator=( const MgcpParameterNumEndPoints& );

        ///
        int num;

        ///
        char st[MAX_ENDPOINT_ID_LIST];
};



/***********************************************************************
 
			      Digit Map
 
**********************************************************************/


///
class MgcpParameterDigitMap : public MgcpParameter
{
    public:
        ///
        MgcpParameterDigitMap()
        {}

        ///
        MgcpParameterDigitMap(const string& x) 
	  : myDigitMap(x)
        {}
        
        ///
        MgcpParameterDigitMap(const MgcpParameterDigitMap& f) 
	  : myDigitMap(f.myDigitMap)
        {}
        
        ///
        ~MgcpParameterDigitMap()
        {}

        ///
        MgcpParameterDigitMap& operator=( const MgcpParameterDigitMap& other )
        {
            myDigitMap = other.myDigitMap;
	    return( *this );
        }

        ///
        string encode();

        ///
        string encodeEmbedded();

        ///
        ostream& printData(ostream& s) const
        {
            return s << myDigitMap;
        }

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        bool parseEmbedded(const string& digitMap);

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterDigitMap(*this);
        }

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeDigitMap;
        }

        ///
        string getDecoded()
        {
            return myDigitMap;
        }

    private:
      
        ///
        string myDigitMap;
};


/****************************************************
QuarantineHandling
*****************************************************/
///
class MgcpParameterQuarantineHandling : public MgcpParameter
{
    public:
        ///
        MgcpParameterQuarantineHandling()
	  :processQuarantine( true ), multipleNotificationsLoop( false )
        {}

        ///
        MgcpParameterQuarantineHandling(bool process, bool loop)
	  :processQuarantine(process), multipleNotificationsLoop(loop)
        {}
    

        ///
        MgcpParameterQuarantineHandling(MgcpNotificationHandling handling,
                                        MgcpQuarantineTreatment treatment)
        {
            if (treatment == MgcpQuarantineTreatmentProcess)
            {
                processQuarantine = true;
            }
            else
            {
                processQuarantine = false;
            }

            if (handling == MgcpNotificationHandlingLoop)
            {
                multipleNotificationsLoop = true;
            }
            else
            {
                multipleNotificationsLoop = false;
            }
        }

        ///
        MgcpParameterQuarantineHandling(const MgcpParameterQuarantineHandling& f) :
        processQuarantine(f.processQuarantine),
        multipleNotificationsLoop(f.multipleNotificationsLoop)
        {}
        
        ///
        ~MgcpParameterQuarantineHandling()
        {}

        ///
        string encode();

        ///
        bool parse( const ParameterSplitType& parms );

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterQuarantineHandling(*this);
        }
        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeQuarantineHandling;
        }

        ///
        MgcpQuarantineTreatment getTreatmentQuarantineHandling();

        ///
        MgcpNotificationHandling getNotificationQuarantineHandling();

    private:
        /// only declared, not defined
        MgcpParameterQuarantineHandling& operator=( const MgcpParameterQuarantineHandling& );

        ///
        bool processQuarantine;
        ///
        bool multipleNotificationsLoop;
};


/************************************************************
LocalConnectionOptions
**************************************************************/
///
class MgcpParameterLocalConnectionOptions : public MgcpParameter
{
    public:
        ///
        MgcpParameterLocalConnectionOptions()
        {}


        ///
        MgcpParameterLocalConnectionOptions(const MgcpConnectionOptions& x)
        : myConnectionOptions(x)
        { }

        ///
        MgcpParameterLocalConnectionOptions(const MgcpParameterLocalConnectionOptions& x)
        : myConnectionOptions(x.myConnectionOptions)
        { }

        ///
        ~MgcpParameterLocalConnectionOptions()
        {}

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeLocalConnectionOptions;
        }

        ///
        string encode();

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterLocalConnectionOptions(*this);
        }

        ///
        MgcpConnectionOptions getDecoded()
        {
            return myConnectionOptions;
        }

    private:
        /// only declared, not defined
        MgcpParameterLocalConnectionOptions& operator=( const MgcpParameterLocalConnectionOptions& );

        ///
        MgcpConnectionOptions myConnectionOptions;
};


///
typedef enum MgcpParameterConnectionModeEnum
{
    ConnectionInvalid,
    ConnectionSendOnly,
    ConnectionRecvOnly,
    ConnectionSendRecv,
    ConnectionConference,
    ConnectionInactive,
    ConnectionLoopback,
    ConnectionContTest,
    ConnectionNetwLoop,
    ConnectionNetwTest,
    ConnectionData
}MgcpParameterConnectionModeEnum;


///
class MgcpParameterConnectionMode : public MgcpParameter
{
    public:
        ///
        MgcpParameterConnectionMode() 
	  : myMode(ConnectionInvalid)
        {}
        
        ///
        MgcpParameterConnectionMode(const string& mode);

        ///
        MgcpParameterConnectionMode(MgcpParameterConnectionModeEnum mode) : myMode(mode)
        {}
        
        ///
        MgcpParameterConnectionMode(const MgcpParameterConnectionMode& f) : myMode(f.myMode)
        {}
        
        ///
        ~MgcpParameterConnectionMode()
        {}

        ///
        string encode();

        ///
        ostream& printData(ostream& s) const;

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterConnectionMode(*this);
        }

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        MgcpParameterConnectionModeEnum getMode()
        {
            return myMode;
        }

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeConnectionMode;
        }

        ///
        string getDecoded();

    private:
        /// only declared, not defined
        MgcpParameterConnectionMode& operator=( const MgcpParameterConnectionMode& );

        ///
        MgcpParameterConnectionModeEnum myMode;
};

/***********************************************************************
 
		   ConnectionDescriptors (see SDP)
 
**********************************************************************/

///
class MgcpParameterLocalConnectionDescriptor : public MgcpParameter
{
    public:
        ///
        MgcpParameterLocalConnectionDescriptor()
        {}
        
        ///
        MgcpParameterLocalConnectionDescriptor(const SdpSession& descriptor) :
        myDescriptor(descriptor)
        {}
        
        ///
        MgcpParameterLocalConnectionDescriptor(const MgcpParameterLocalConnectionDescriptor& x) 
	  : myDescriptor(x.myDescriptor)
        {}
        
        ///
        ~MgcpParameterLocalConnectionDescriptor()
        {}

        ///
        string encode()
        {
            return myDescriptor.encode();
        }

        ///
        SdpSession* getDescriptor()
        {
            return &myDescriptor;
        }

        ///
        ostream& printData(ostream& s) const
        {
            string x;
            x = static_cast < SdpSession > (myDescriptor).encode();
            return s << x << "\n";
        }
        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterLocalConnectionDescriptor(*this);
        }
        ///
        MgcpParameterLocalConnectionDescriptor& operator= (const MgcpParameterLocalConnectionDescriptor& id)
        {
            this->myDescriptor = id.myDescriptor;
            return *this;
        }

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeLocalConnectionDescriptor;
        }

        ///
        SdpSession getDecoded()
        {
            return myDescriptor;
        }

    protected:
        ///
        SdpSession myDescriptor;
};


///
class MgcpParameterRemoteConnectionDescriptor : public MgcpParameter
{
    public:
        ///
        MgcpParameterRemoteConnectionDescriptor()
        {}
        
        ///
        MgcpParameterRemoteConnectionDescriptor(const SdpSession& descriptor) :
        myDescriptor(descriptor)
        {}
        
        ///
        MgcpParameterRemoteConnectionDescriptor(const MgcpParameterRemoteConnectionDescriptor& x) :
        myDescriptor(x.myDescriptor)
        {}
        
        ///
        ~MgcpParameterRemoteConnectionDescriptor()
        {}

        ///
        string encode()
        {
            return myDescriptor.encode();
        }

        ///
        SdpSession* getDescriptor()
        {
            return &myDescriptor;
        }

        ///
        ostream& printData(ostream& s) const
        {
            string x;
            x = static_cast < SdpSession > (myDescriptor).encode();
            return s << x << "\n";
        }

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeRemoteConnectionDescriptor;
        }

        ///
        SdpSession getDecoded()
        {
            return myDescriptor;
        }


    protected:
        /// only declared, not defined
        MgcpParameterRemoteConnectionDescriptor& operator=( const MgcpParameterRemoteConnectionDescriptor& );

        ///
        SdpSession myDescriptor;
};


/// Method used in RestartInProgress
class MgcpParameterRestartMethod : public MgcpParameter
{
    public:
        ///
        MgcpParameterRestartMethod()
        {}

        ///
        MgcpParameterRestartMethod(const string& method)
	  : myMethod( method )
        {}

        ///
        MgcpParameterRestartMethod(const MgcpParameterRestartMethod& x)
	  : myMethod( x.myMethod )
        {}

        ///
        ~MgcpParameterRestartMethod()
        {}

        ///
        string encode();

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeRestartMethod;
        }

        ///
        string getDecoded() const
        {
            return myMethod;
        }


    private:
        /// only declared, not defined
        MgcpParameterRestartMethod& operator=( const MgcpParameterRestartMethod& );

        ///
        string myMethod;

};


/// Method used in RestartInProgress
class MgcpParameterRestartDelay : public MgcpParameter
{
    public:
        ///
        MgcpParameterRestartDelay()
        {}

        ///
        MgcpParameterRestartDelay(const string& delay)
	  : myMethod( delay )
        {}

        ///
        MgcpParameterRestartDelay(const MgcpParameterRestartMethod& x)
        {
	     const string y = x.getDecoded();
	     myMethod = y;
	     //  myMethod = x.getDecoded();
	}

        ///
        ~MgcpParameterRestartDelay()
        {}

        ///
        string encode();

        ///
        bool parse(const ParameterSplitType& parms);

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeRestartDelay;
        }

        ///
        string getDecoded()
        {
            return myMethod;
        }

    private:
        /// only declared, not defined
        MgcpParameterRestartDelay& operator=( const MgcpParameterRestartDelay& );

        ///
        string myMethod;

};


class MgcpParameterEventStates : public MgcpParameter
{
    public:
        ///
        MgcpParameterEventStates()
        {}
          
        ///
        MgcpParameterEventStates(const MgcpParameterEventStates& e) 
	  : list(e.list)
        {}
        
        ///
        ~MgcpParameterEventStates()
        {
	    list.clear();
	}
        
        ///
        string encode();
        ///
        bool insert(Sptr < MgcpParameterSignal > event);
        ///
        ostream& printData(ostream& s) const;
        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterEventStates(*this);
        }
        ///
        bool parse(const ParameterSplitType& parms);
        ///
        vector < Sptr < MgcpParameterSignal > > * getlist()
        {
            return &list;
        }
        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeEventStates;
        }
        ///
        int getDecodedNum()
        {
            return list.size();
        }
        ///
        MgcpParameterSignal* getDecodedItem(int i)
        {
            assert( static_cast < unsigned int > (i) < list.size() );
            return list[i].getPtr();
        }

    private:
        /// only declared, not defined
        MgcpParameterEventStates& operator=( const MgcpParameterEventStates& );

        ///
        vector < Sptr < MgcpParameterSignal > > list;
};



class MgcpParameterCapabilities : public MgcpParameter
{
    public:
        ///
        MgcpParameterCapabilities()
	  : myConnectionOptions()
        {}
        

        ///
        MgcpParameterCapabilities(MgcpConnectionOptions& options, vector < MgcpParameterConnectionModeEnum > connmode, vector < MgcpPackageType > packlist) 
	  : myConnectionOptions(options), ConnectionModeList(connmode), EventPackageList(packlist)
        {}
        

        ///
        MgcpParameterCapabilities(const MgcpParameterCapabilities& x)
        : myConnectionOptions( x.myConnectionOptions), 
          ConnectionModeList( x.ConnectionModeList), 
          EventPackageList( x.EventPackageList)
        {}

        ///
        ~MgcpParameterCapabilities()
        {
            ConnectionModeList.clear();
            EventPackageList.clear();
	}
        

        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeCapabilities;
        }
        ///
        string PacketizationEncode();
        ///
        string BandwidthEncode();
        ///
        string CompressionEncode();
        ///
        string EventPackageEncode();
        ///
        string ConnectionModeEncode();
        ///
        string EchoCancellationEncode();
        ///
        string TypeofServiceEncode();
        ///
        string SilenceSuppressionEncode();
        ///
        string encode();
        ///
        bool parse(const ParameterSplitType& parms);
        ///
        bool ConnectionModeParse(const ParameterSplitType& parameter);
        ///
        bool EventPackageParse(const ParameterSplitType& parameter);
        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterCapabilities(*this);
        }
        ///
        int
        getDecoded()
        {
            return myConnectionOptions.getPacketization();
        }
        ///
        void setPacketization(int value, int val = 0)
        {
            myConnectionOptions.setPacketization(value, val);
        }
        ///
        bool isPacketizationRange()
        {
            return myConnectionOptions.isPacketizationRange();
        }
        ///
        int getPacketization()
        {
            return myConnectionOptions.getPacketization();
        }
        ///
        void getPacketizationRange(int* p1, int* p2)
        {
            return myConnectionOptions.getPacketizationRange(p1, p2);
        }
        ///
        void setBandwidth(int value, int val = 0)
        {
            myConnectionOptions.setBandwidth(value, val);
        }
        ///
        bool isBandwidthRange()
        {
            return myConnectionOptions.isBandwidthRange();
        }
        ///
        int getBandwidth()
        {
            return myConnectionOptions.getBandwidth();
        }
        ///
        ///
        void getBandwidthRange(int* p1, int* p2)
        {
            return myConnectionOptions.getBandwidthRange(p1, p2);
        }
        ///
        void setSilenceSuppression(bool value)
        {
            myConnectionOptions.setSilenceSuppression(value);
        }
        ///
        bool getSilenceSuppression()
        {
            return myConnectionOptions.getSilenceSuppression();
        }
        ///
        void setEchoCancellation(bool value)
        {
            myConnectionOptions.setEchoCancellation(value);
        }
        ///
        bool getEchoCancellation()
        {
            return myConnectionOptions.getEchoCancellation();
        }
        ///
        void setTypeofService(int value)
        {
            myConnectionOptions.setTypeofService(value);
        }
        ///
        int getTypeofService()
        {
            return myConnectionOptions.getTypeofService();
        }
        ///
        void setCompressionAlgo( string value)
        {
            myConnectionOptions.setCompressionAlgo(value);
        }
        ///
        void setCompressionAlgoList( vector < string > algolist)
        {
            myConnectionOptions.setCompressionAlgoList(algolist);
        }
        ///
        string getCompressionAlgo()
        {
            return ( myConnectionOptions.getCompressionAlgo() );
        }
        ///
        int getCompressionAlgoNum()
        {
            return myConnectionOptions.getCompressionAlgoNum();
        }
        ///
        string getCompressionAlgoItem( int i)
        {
            return myConnectionOptions.getCompressionAlgoItem(i);
        }


        ///
        bool setConnectionMode(const string& mode);

        ///
        int getConnectionModeNum()
        {
            return ConnectionModeList.size();
        }
        ///
        string getConnectionModeItem(int i);

        ///
        void setEventPackage(MgcpPackageType mytype)
        {
            EventPackageList.push_back(mytype);
        }
        ///
        int getEventPackageNum()
        {
            return EventPackageList.size();
        }
        ///
        MgcpPackageType getEventPackageItem(int i)
        {
            assert( static_cast < unsigned int > (i) < EventPackageList.size() );
            return EventPackageList[i];
        }

    private:
        /// only declared, not defined
        MgcpParameterCapabilities& operator=( const MgcpParameterCapabilities& );

        ///
        MgcpConnectionOptions myConnectionOptions;
        vector < MgcpParameterConnectionModeEnum > ConnectionModeList;
        vector < MgcpPackageType > EventPackageList;

};

/*********************************************************
           Connection Parameters
**********************************************************/

class MgcpParameterConnectionParameters: public MgcpParameter
{
    public:
        ///
        MgcpParameterConnectionParameters() 
	  : packetssent(0), octetssent(0), packetsrcvd(0), octetsrcvd(0),
        packetslost(0), jitter(0), latency(0)
        {}
        
        ///
        MgcpParameterConnectionParameters(int pktsent, int octsent, int pktrcvd, int octrcvd, int pktlost,
                                          int jitt, int laten)
        {
            packetssent = pktsent;
            octetssent = octsent;
            packetsrcvd = pktrcvd;
            octetsrcvd = octrcvd;
            packetslost = pktlost;
            jitter = jitt;
            latency = laten;
        }

        ///
        MgcpParameterConnectionParameters(const MgcpParameterConnectionParameters& myparameter)
        {
            CopyElements( myparameter );
        }

        ///
        ~MgcpParameterConnectionParameters()
        {}
        
        ///
        string encode();
        ///
        //  ostream& printData(ostream& s) const;

        ///
        MgcpParameter* duplicate()
        {
            return new MgcpParameterConnectionParameters(*this);
        }
        ///
        int operator== (const MgcpParameterConnectionParameters& myparameter) const;
        ///
        bool parse(const ParameterSplitType& parms);
        ///
        MgcpParameterType type()
        {
            return MgcpParameterTypeConnectionParameters;
        }
        ///
        void CopyElements(const MgcpParameterConnectionParameters& myparameter);
        ///
        string itos(int value);
        ///
        void setPacketsSent(int ps);
        ///
        int getPacketsSent();
        ///
        void setOctetsSent(int os);
        ///
        int getOctetsSent();
        ///
        void setPacketsRcvd(int pr);
        ///
        int getPacketsRcvd();
        ///
        void setOctetsRcvd(int r);
        ///
        int getOctetsRcvd();
        ///
        void setPacketsLost(int pl);
        ///
        int getPacketsLost();
        ///
        void setJitter(int ji);
        ///
        int getJitter();
        ///
        void setLatency(int la);
        ///
        int getLatency();
        ///

    private:
        /// only declared, not defined
        MgcpParameterConnectionParameters& operator=( const MgcpParameterConnectionParameters& );

        ///
        int packetssent;
        int octetssent;
        int packetsrcvd;
        int octetsrcvd;
        int packetslost;
        int jitter;
        int latency;
};

#endif
